@@ -28,8 +28,6 @@ or
 
 =head1 LEGAL NOTICE
 
-=for comment help for syntax-highlighting editor that does not understand POD "
-
 The following two paragraphs are quoted from the Space Track web site.
 
 Due to existing National Security Restrictions pertaining to access of
@@ -88,7 +86,7 @@ package Astro::SpaceTrack;
 
 use base qw{Exporter};
 
-our $VERSION = '0.026';
+our $VERSION = '0.027';
 our @EXPORT_OK = qw{shell BODY_STATUS_IS_OPERATIONAL BODY_STATUS_IS_SPARE
     BODY_STATUS_IS_TUMBLING};
 our %EXPORT_TAGS = (
@@ -160,6 +158,10 @@ my %catalogs = (	# Catalog names (and other info) for each source.
 	cubesat => {name => 'CubeSats'},
 	other => {name => 'Other'},
 	},
+    iridium_status => {
+	kelso => {name => 'Celestrak (Kelso)'},
+	mccants => {name => 'McCants'},
+    },
     spaceflight => {
 	iss => {name => 'International Space Station',
 		url => 'http://spaceflight.nasa.gov/realdata/sightings/SSapplications/Post/JavaSSOP/orbit/ISS/SVPOST.html',
@@ -189,6 +191,7 @@ my %mutator = (	# Mutators for the various attributes.
     addendum => \&_mutate_attrib,		# Addendum to banner text.
     banner => \&_mutate_attrib,
     cookie_expires => \&_mutate_attrib,
+    debug_url => \&_mutate_attrib,	# Force the URL. Undocumented and unsupported.
     direct => \&_mutate_attrib,
     dump_headers => \&_mutate_attrib,	# Dump all HTTP headers. Undocumented and unsupported.
     filter => \&_mutate_attrib,
@@ -261,6 +264,7 @@ my $self = {
     agent => LWP::UserAgent->new (),
     banner => 1,	# shell () displays banner if true.
     cookie_expires => undef,
+    debug_url => undef,	# Not turned on
     dump_headers => 0,	# No dumping.
     filter => 0,	# Filter mode.
     iridium_status_format => 'mccants',	# For historical reasons.
@@ -308,8 +312,6 @@ return $self;
 
 =item $resp = $st->amsat ()
 
-=for comment Help syntax-highlighting editor. "
-
 This method downloads current orbital elements from the Radio Amateur
 Satellite Corporation's web page, L<http://www.amsat.org/>. This lists
 satellites of interest to radio amateurs, and appears to be updated
@@ -322,8 +324,6 @@ the setting of the 'with_name' attribute is ignored.
 This method is a web page scraper. any change in the location of the
 web page will break this method.
 
-=for comment Help syntax-highlighting editor. "
-
 =cut
 
 sub amsat {
@@ -370,14 +370,10 @@ sub attribute_names {wantarray ? sort keys %mutator : [sort keys %mutator]}
 
 =item $resp = banner ();
 
-=for comment help for syntax-highlighting editor "
-
 This method is a convenience/nuisance: it simply returns a fake
 HTTP::Response with standard banner text. It's really just for the
 benefit of the shell method.
 
-=for comment help for syntax-highlighting editor "
-
 =cut
 
 sub banner {
@@ -393,7 +389,7 @@ and you must abide by that site's restrictions, which include
 not making the data available to a third party without prior
 permission.
 
-Copyright 2005, 2006 T. R. Wyant (wyant at cpan dot org). All
+Copyright 2005, 2006, 2007 T. R. Wyant (wyant at cpan dot org). All
 rights reserved.
 
 This module is free software; you can use it, redistribute it
@@ -418,11 +414,12 @@ list reference to list references  (i.e. a list of lists). Each
 of the list references contains the catalog ID of a satellite or
 other orbiting body and the common name of the body.
 
-If the 'direct' attribute is true, the elements will be fetched
-directly from Celestrak, and no login is needed. Otherwise, This
-method implicitly calls the login () method if the session cookie
-is missing or expired. If login () fails, you will get the
-HTTP::Response from login ().
+If the 'direct' attribute is true, the elements will be fetched directly
+from Celestrak, and no login is needed. Otherwise, This method
+implicitly calls the login () method if the session cookie is missing or
+expired, and returns the SpaceTrack data for the OIDs fetched from
+Celestrak. If login () fails, you will get the HTTP::Response from login
+().
 
 If this method succeeds, a 'Pragma: spacetrack-type = orbit' header is
 added to the HTTP::Response object returned.
@@ -545,8 +542,6 @@ return $self->_handle_observing_list ($opt, <$fh>)
 
 =item $resp = $st->get (attrib)
 
-=for comment help syntax-highlighting editor "
-
 B<This method returns an HTTP::Response object> whose content is the value
 of the given attribute. If called in list context, the second element
 of the list is just the value of the attribute, for those who don't want
@@ -554,8 +549,6 @@ to winkle it out of the response object. We croak on a bad attribute name.
 
 See L</Attributes> for the names and functions of the attributes.
 
-=for comment help syntax-highlighting editor "
-
 =cut
 
 sub get {
@@ -577,8 +570,6 @@ return wantarray ? ($resp, $self->{$name}) : $resp;
 
 =item $resp = $st->help ()
 
-=for comment help syntax-highlighting editor "
-
 This method exists for the convenience of the shell () method. It
 always returns success, with the content being whatever it's
 convenient (to the author) to include.
@@ -586,8 +577,6 @@ convenient (to the author) to include.
 If the L<webcmd|/webcmd> attribute is set, the L<http://search.cpan.org/>
 web page for this version of Astro::Satpass is launched.
 
-=for comment help syntax-highlighting editor "
-
 =cut
 
 sub help {
@@ -636,6 +625,8 @@ The following commands are defined:
       addendum = extra text for the shell () banner;
       banner = false to supress the shell () banner;
       cookie_expires = Perl date the session cookie expires;
+      debug_url = Override canned url for debugging - do not
+        set this in normal use;
       direct = true to fetch orbital elements directly
         from a redistributer. Currently this only affects the
         celestrak() method. The default is false.
@@ -676,17 +667,17 @@ eod
 
 =for html <a name="iridium_status"></a>
 
-=item $resp = $st->iridium_status ();
-
-=for comment help syntax-highlighting editor "
+=item $resp = $st->iridium_status ($format);
 
 This method queries its sources of Iridium status, returning an
 HTTP::Response object containing the relevant data (if all queries
 succeeded) or the status of the first failure. If the queries succeed,
 the content is a series of lines formatted by "%6d   %-15s%-8s %s\n",
-with NORAD ID, name, status, and comment substituted in. What actually
-appears in the status and comment depends on the contents of the
-L</iridium_status_format> attribute as follows:
+with NORAD ID, name, status, and comment substituted in.
+
+The source of the data and, to a certain extent, the format of the
+results is determined by the optional $format argument, which defaults
+to the value of the L</iridium_status_format> attribute.
 
 If the format is 'kelso', only celestrak.com is queried for the
 data. The possible status values are:
@@ -740,8 +731,6 @@ tumbling.
 
 The BODY_STATUS constants are exportable using the :status tag.
 
-=for comment help syntax-highlighting editor "
-
 =cut
 
 {	# Begin local symbol block.
@@ -782,7 +771,7 @@ The BODY_STATUS constants are exportable using the :status tag.
 
     sub iridium_status {
     my $self = shift;
-    my $fmt = $self->{iridium_status_format};
+    my $fmt = shift || $self->{iridium_status_format};
     delete $self->{_content_type};
     my %rslt;
     my $kelso_url = $self->get ('url_iridium_status_kelso')->content;
@@ -843,8 +832,6 @@ The BODY_STATUS constants are exportable using the :status tag.
 
 =item $resp = $st->login ( ... )
 
-=for comment help syntax-highlighting editor "
-
 If any arguments are given, this method passes them to the set ()
 method. Then it executes a login. The return is normally the
 HTTP::Response object from the login. But if no session cookie was
@@ -854,8 +841,6 @@ a login is attempted without the username and password being set, the
 return is an HTTP::Response with an appropriate message and the
 code set to RC_PRECONDITION_FAILED from HTTP::Status (a.k.a. 412).
 
-=for comment help syntax-highlighting editor "
-
 =cut
 
 sub login {
@@ -896,16 +881,12 @@ HTTP::Response->new (RC_OK, undef, undef, "Login successful.\n");
 
 =item $resp = $st->names (source)
 
-=for comment help syntax-highlighting editor "
-
 This method retrieves the names of the catalogs for the given source,
 either 'celestrak' or 'spacetrack', in the content of the given
 HTTP::Response object. In list context, you also get a reference to
 a list of two-element lists; each inner list contains the description
 and the catalog name (suitable for inserting into a Tk Optionmenu).
 
-=for comment help syntax-highlighting editor "
-
 =cut
 
 sub names {
@@ -934,8 +915,9 @@ return ($resp, \@list);
 =item $resp = $st->retrieve (number_or_range ...)
 
 This method retrieves the latest element set for each of the given
-catalog numbers. Non-numeric catalog numbers are ignored, as are
-(at a later stage) numbers that do not actually represent a satellite.
+satellite ID numbers (also known as SATCAT IDs, NORAD IDs, or OIDs).
+Non-numeric catalog numbers are ignored, as are (at a later stage)
+numbers that do not actually represent a satellite.
 
 Number ranges are represented as 'start-end', where both 'start' and
 'end' are catalog numbers. If 'start' > 'end', the numbers will be
@@ -1076,15 +1058,53 @@ $resp;
 
 =item $resp = $st->search_date (date ...)
 
-=for comment help syntax-highlighting editor "
-
 This method searches the database for objects launched on the given
 date. The date is specified as year-month-day, with any non-digit being
 legal as the separator. You can omit -day or specify it as 0 to get
 all launches for the given month. You can omit -month (or specify it
-as 0) as well to get all launches for the given year. There is no
-mechanism to restrict the search to a given date range, on-orbit
-status, or to filter out debris or rocket bodies.
+as 0) as well to get all launches for the given year.
+
+You can specify options for the search as either command-type options
+(e.g. search (-status => 'onorbit', ...)) or as a leading hash reference
+(e.g. search ({status => onorbit}, ...)). If you specify the hash
+reference, option names must be specified in full, without the leading
+'-', and the argument list will not be parsed for command-type options.
+Options that take multiple values (i.e. 'exclude') must have their
+values specified as a hash reference, even if you only specify one value
+- or none at all.
+
+If you specify command-type options, they may be abbreviated, as long as
+the abbreviation is unique. Errors in either sort of specification
+result in an exception being thrown.
+
+In addition to the options available for L</retrieve>, the following
+options may be specified:
+
+ exclude
+   specifies the types of bodies to exclude. The
+   value is one or more of 'debris' or 'rocket'.
+   If you specify both as command-style options,
+   you may either specify the option more than once,
+   or specify the values comma-separated.
+ status
+   specifies the desired status of the returned body
+   (or bodies). Must be 'onorbit', 'decayed', or 'all'.
+   The default is 'all'. Note that this option
+   represents status at the time the search was done;
+   you can not combine it with the retrieve() date
+   options to find bodies onorbit as of a given date
+   in the past.
+
+Examples:
+
+ search_date (-status => 'onorbit', -exclude =>
+    'debris,rocket', -last5 '2005-12-25');
+ search_date (-exclude => 'debris',
+    -exclude => 'rocket', '2005/12/25');
+ search_date ({exclude => ['debris', 'rocket']},
+    '2005-12-25');
+ search_date ({exclude => 'debris,rocket'}, # INVALID!
+    '2005-12-25');
 
 This method implicitly calls the login () method if the session cookie
 is missing or expired. If login () fails, you will get the
@@ -1101,16 +1121,13 @@ for each match.
 If this method succeeds, a 'Pragma: spacetrack-type = orbit' header is
 added to the HTTP::Response object returned.
 
-You can specify the L</retrieve> options on this method as well.
-
-=for comment help syntax-highlighting editor "
-
 =cut
 
 sub search_date {
 my $self = shift;
+@_ = _parse_search_args (@_);
 $self->_search_generic (sub {
-    my ($self, $name) = @_;
+    my ($self, $name, $opt) = @_;
     my ($year, $month, $day) =
 	$name =~ m/^(\d+)(?:\D+(\d+)(?:\D+(\d+))?)?/
 	    or return undef;
@@ -1122,8 +1139,8 @@ $self->_search_generic (sub {
 	launch_year => $year,
 	launch_month => $month,
 	launch_day => $day,
-	status => 'all',	# or 'onorbit' or 'decayed'.
-##	exclude => '',		# or 'debris' or 'rocket' or both.
+	status => $opt->{status},	# 'all', 'onorbit' or 'decayed'.
+	exclude => $opt->{exclude},	# ['debris', 'rocket', or both]
 	_sessionid => '',
 	_submit => 'submit',
 	_submitted => 1,
@@ -1136,18 +1153,15 @@ $self->_search_generic (sub {
 
 =item $resp = $st->search_id (id ...)
 
-=for comment help syntax-highlighting editor "
-
 This method searches the database for objects having the given
-international IDs. The international ID is the last two digits
-of the launch year (in the range 1957 through 2056), the
-three-digit sequence number of the launch within the year (with
-leading zeroes as needed), and the piece (A through ZZ, with A
-typically being the payload). You can omit the piece and get all
-pieces of that launch, or omit both the piece and the launch
-number and get all launches for the year. There is no
-mechanism to restrict the search to a given date range, on-orbit
-status, or to filter out debris or rocket bodies.
+international IDs. The international ID is the last two digits of the
+launch year (in the range 1957 through 2056), the three-digit sequence
+number of the launch within the year (with leading zeroes as needed),
+and the piece (A through ZZ, with A typically being the payload). You
+can omit the piece and get all pieces of that launch, or omit both the
+piece and the launch number and get all launches for the year. There is
+no mechanism to restrict the search to a given on-orbit status, or to
+filter out debris or rocket bodies.
 
 This method implicitly calls the login () method if the session cookie
 is missing or expired. If login () fails, you will get the
@@ -1155,9 +1169,9 @@ HTTP::Response from login ().
 
 On success, this method returns an HTTP::Response object whose content
 is the relevant element sets. If called in list context, the first
-element of the list is the aforementioned HTTP::Response object, and
-the second element is a list reference to list references  (i.e. a list
-of lists). The first list reference contains the header text for all
+element of the list is the aforementioned HTTP::Response object, and the
+second element is a list reference to list references  (i.e. a list of
+lists). The first list reference contains the header text for all
 columns returned, and the subsequent list references contain the data
 for each match.
 
@@ -1166,14 +1180,13 @@ added to the HTTP::Response object returned.
 
 You can specify the L</retrieve> options on this method as well.
 
-=for comment help syntax-highlighting editor "
-
 =cut
 
 sub search_id {
 my $self = shift;
+## @_ = _parse_search_args (@_);
 $self->_search_generic (sub {
-    my ($self, $name) = @_;
+    my ($self, $name, $opt) = @_;
     my ($year, $number, $piece) =
 	$name =~ m/^(\d\d)(\d{3})?([[:alpha:]])?$/ or return undef;
     $year += $year < 57 ? 2000 : 1900;
@@ -1182,8 +1195,8 @@ $self->_search_generic (sub {
 	launch_year => $year,
 	launch_number => $number || '',
 	piece => uc ($piece || ''),
-	status => 'all',	# or 'onorbit' or 'decayed'.
-##	exclude => '',		# or 'debris' or 'rocket' or both.
+	status => $opt->{status},	# 'all', 'onorbit' or 'decayed'.
+	exclude => $opt->{exclude},	# ['debris', 'rocket', or both]
 	_sessionid => '',
 	_submit => 'submit',
 	_submitted => 1,
@@ -1196,12 +1209,8 @@ $self->_search_generic (sub {
 
 =item $resp = $st->search_name (name ...)
 
-=for comment help syntax-highlighting editor "
-
 This method searches the database for the named objects. Matches
-are case-insensitive and all matches are returned. There is no
-mechanism to restrict the search to a given date range, on-orbit
-status, or to filter out debris or rocket bodies.
+are case-insensitive and all matches are returned.
 
 This method implicitly calls the login () method if the session cookie
 is missing or expired. If login () fails, you will get the
@@ -1218,25 +1227,26 @@ for each match.
 If this method succeeds, a 'Pragma: spacetrack-type = orbit' header is
 added to the HTTP::Response object returned.
 
-You can specify the L</retrieve> options on this method as well.
-
-=for comment help syntax-highlighting editor "
+You can specify the L</retrieve> and L</search_date> options on this
+method as well. The L</search_date> -status option is known to work,
+but I am not sure about the efficacy the -exclude option.
 
 =cut
 
 sub search_name {
 my $self = shift;
+@_ = _parse_search_args (@_);
 $self->_search_generic (sub {
-    my ($self, $name) = @_;
+    my ($self, $name, $opt) = @_;
     $self->_post ('perl/name_query.pl',
+	_submitted => 1,
+	_sessionid => '',
 	name => $name,
 	launch_year_start => 1957,
 	launch_year_end => (gmtime)[5] + 1900,
-	status => 'all',	# or 'onorbit' or 'decayed'.
-##	exclude => '',		# or 'debris' or 'rocket' or both.
-	_sessionid => '',
+	status => $opt->{status},	# 'all', 'onorbit' or 'decayed'.
+	exclude => $opt->{exclude},	# ['debris', 'rocket', or both]
 	_submit => 'Submit',
-	_submitted => 1,
 	);
     }, @_);
 }
@@ -1246,8 +1256,6 @@ $self->_search_generic (sub {
 
 =item $st->set ( ... )
 
-=for comment help syntax-highlighting editor "
-
 This is the mutator method for the object. It can be called explicitly,
 but other methods as noted may call it implicitly also. It croaks if
 you give it an odd number of arguments, or if given an attribute that
@@ -1259,8 +1267,6 @@ error we croak.
 
 See L</Attributes> for the names and functions of the attributes.
 
-=for comment help syntax-highlighting editor "
-
 =cut
 
 sub set {
@@ -1284,8 +1290,6 @@ HTTP::Response->new (RC_OK, undef, undef, COPACETIC);
 
 =item $st->shell ()
 
-=for comment help syntax-highlighting editor "
-
 This method implements a simple shell. Any public method name except
 'new' or 'shell' is a command, and its arguments if any are parameters.
 We use Text::ParseWords to parse the line, and blank lines or lines
@@ -1317,8 +1321,6 @@ one of the arguments was 'exit'.
 
 Unlike most of the other methods, this one returns nothing.
 
-=for comment help syntax-highlighting editor "
-
 =cut
 
 my ($read, $print, $out, $rdln);
@@ -1431,8 +1433,6 @@ $self->shell ($self->_source (@_), 'exit');
 
 =item $resp = $st->spaceflight ()
 
-=for comment help syntax-highlighting editor "
-
 This method downloads current orbital elements from NASA's human
 spaceflight site, L<http://spaceflight.nasa.gov/>. As of July 2006
 you get the International Space Station, and the current Space Shuttle
@@ -1450,8 +1450,6 @@ You can specify the L</retrieve> options on this method as well. In
 addition, you can specify -all to get all data. -all will be ignored
 if -start_epoch, -end_epoch, or -last5 is specified.
 
-=for comment help syntax-highlighting editor "
-
 =cut
 
 
@@ -1808,7 +1806,8 @@ goto &_mutate_attrib;
 
 sub _mutate_iridium_status_format {
 croak "Error - Illegal status format '$_[2]'"
-    unless $_[2] eq 'kelso' || $_[2] eq 'mccants';
+##    unless $_[2] eq 'kelso' || $_[2] eq 'mccants';
+    unless $catalogs{iridium_status}{$_[2]};
 $_[0]->{$_[1]} = $_[2];
 }
 
@@ -1931,6 +1930,51 @@ eod
 $opt;
 }
 
+#	_parse_search_args parses the search_*() options off its
+#	arguments, prefixes a reference to the resultant options
+#	hash to the remaining arguments, and returns the resultant
+#	list. If the first argument is a hash reference, it simply
+#	returns its argument list, under the assumption that it
+#	has already been called.
+
+my @legal_search_args = (
+    'status=s' => q{('onorbit', 'decayed', or 'all')},
+    'exclude=s@' => q{('debris', 'rocket', or 'debris,rocket')},
+);
+my %legal_search_exclude = map {$_ => 1} qw{debris rocket};
+my %legal_search_status = map {$_ => 1} qw{onorbit decayed all};
+sub _parse_search_args {
+unless (ref ($_[0]) eq 'HASH') {
+    ref $_[0] eq 'ARRAY' and my @extra = @{shift @_};
+    @_ = _parse_retrieve_args ([@legal_search_args, @extra], @_);
+
+##    my %lgl = (@legal_search_args, ref $_[0] eq 'ARRAY' ? @{shift @_} : ());
+##    my $opt = {};
+##    local @ARGV = @_;
+
+##    GetOptions ($opt, keys %lgl) or croak <<eod;
+##Error - Legal options are@{[map {(my $q = $_) =~ s/=.*//;
+##	"\n  -$q $lgl{$_}"} sort keys %lgl]}
+##eod
+    my $opt = $_[0];
+    $opt->{status} ||= 'all',
+    $legal_search_status{$opt->{status}} or croak <<eod;
+Error - Illegal status '$opt->{status}'. You must specify one of
+        @{[join ', ', map {"'$_'"} sort keys %legal_search_status]}
+eod
+    $opt->{exclude} ||= [];
+    $opt->{exclude} = [map {split ',', $_} @{$opt->{exclude}}];
+    foreach (@{$opt->{exclude}}) {
+	$legal_search_exclude{$_} or croak <<eod;
+Error - Illegal exclusion '$_'. You must specify one or more of
+        @{[join ', ', map {"'$_'"} sort keys %legal_search_exclude]}
+eod
+	}
+##    @_ = ($opt, @ARGV);
+    }
+@_;
+}
+
 #	_post is just like _get, except for the method used. DO NOT use
 #	this method in the login () method, or you get a bottomless
 #	recursion.
@@ -1943,9 +1987,10 @@ my $path = shift;
 	my $resp = $self->login ();
 	return $resp unless $resp->is_success;
 	};
-    my $resp = $self->{agent}->post ("http://@{[DOMAIN]}/$path", [@_]);
+    my $url = $self->{debug_url} || "http://@{[DOMAIN]}/$path";
+    my $resp = $self->{agent}->post ($url, [@_]);
     $self->_dump_headers ($resp) if $self->{dump_headers};
-    return $resp unless $resp->is_success;
+    return $resp unless $resp->is_success && !$self->{debug_url};
     local $_ = $resp->content;
     m/login\.pl/i and do {
 	$self->{cookie_expires} = 0;
@@ -1983,7 +2028,7 @@ my @table;
 my %id;
 foreach my $name (@_) {
     defined (my $resp = $poster->($self, $name, $opt)) or next;
-    return $resp unless $resp->is_success;
+    return $resp unless $resp->is_success && !$self->{debug_url};
     my $content = $resp->content;
     next if $content =~ m/No results found/i;
     my @this_page = @{$p->parse_string (table => $content)};
@@ -2316,7 +2361,7 @@ Thomas R. Wyant, III (F<wyant at cpan dot org>)
 
 =head1 COPYRIGHT
 
-Copyright 2005, 2006 by Thomas R. Wyant, III
+Copyright 2005, 2006, 2007 by Thomas R. Wyant, III
 (F<wyant at cpan dot org>). All rights reserved.
 
 This module is free software; you can use it, redistribute it
